@@ -558,7 +558,7 @@ arg_to_gl_blend_factor (CoglBlendStringArgument *arg)
   else if (arg->factor.source.info->type ==
            COGL_BLEND_STRING_COLOR_SOURCE_SRC_COLOR)
     {
-      if (arg->factor.source.mask == COGL_BLEND_STRING_CHANNEL_MASK_RGB)
+      if (arg->factor.source.mask != COGL_BLEND_STRING_CHANNEL_MASK_ALPHA)
         {
           if (arg->factor.source.one_minus)
             return GL_ONE_MINUS_SRC_COLOR;
@@ -576,7 +576,7 @@ arg_to_gl_blend_factor (CoglBlendStringArgument *arg)
   else if (arg->factor.source.info->type ==
            COGL_BLEND_STRING_COLOR_SOURCE_DST_COLOR)
     {
-      if (arg->factor.source.mask == COGL_BLEND_STRING_CHANNEL_MASK_RGB)
+      if (arg->factor.source.mask != COGL_BLEND_STRING_CHANNEL_MASK_ALPHA)
         {
           if (arg->factor.source.one_minus)
             return GL_ONE_MINUS_DST_COLOR;
@@ -595,7 +595,7 @@ arg_to_gl_blend_factor (CoglBlendStringArgument *arg)
   else if (arg->factor.source.info->type ==
            COGL_BLEND_STRING_COLOR_SOURCE_CONSTANT)
     {
-      if (arg->factor.source.mask == COGL_BLEND_STRING_CHANNEL_MASK_RGB)
+      if (arg->factor.source.mask != COGL_BLEND_STRING_CHANNEL_MASK_ALPHA)
         {
           if (arg->factor.source.one_minus)
             return GL_ONE_MINUS_CONSTANT_COLOR;
@@ -646,7 +646,6 @@ cogl_material_set_blend (CoglHandle handle,
 {
   CoglMaterial *material;
   CoglBlendStringStatement statements[2];
-  CoglBlendStringStatement split[2];
   CoglBlendStringStatement *rgb;
   CoglBlendStringStatement *a;
   GError *internal_error = NULL;
@@ -674,13 +673,8 @@ cogl_material_set_blend (CoglHandle handle,
       return FALSE;
     }
 
-  if (statements[0].mask == COGL_BLEND_STRING_CHANNEL_MASK_RGBA)
-    {
-      _cogl_blend_string_split_rgba_statement (statements,
-                                               &split[0], &split[1]);
-      rgb = &split[0];
-      a = &split[1];
-    }
+  if (count == 1)
+    rgb = a = statements;
   else
     {
       rgb = &statements[0];
@@ -1559,6 +1553,19 @@ _cogl_material_flush_layers_gl_state (CoglMaterial *material,
     }
 }
 
+#ifndef HAVE_COGL_GLES
+
+static gboolean
+blend_factor_uses_constant (GLenum blend_factor)
+{
+  return (blend_factor == GL_CONSTANT_COLOR ||
+          blend_factor == GL_ONE_MINUS_CONSTANT_COLOR ||
+          blend_factor == GL_CONSTANT_ALPHA ||
+          blend_factor == GL_ONE_MINUS_CONSTANT_ALPHA);
+}
+
+#endif
+
 static void
 _cogl_material_flush_base_gl_state (CoglMaterial *material,
                                     gboolean skip_gl_color)
@@ -1620,31 +1627,34 @@ _cogl_material_flush_base_gl_state (CoglMaterial *material,
 #endif
 
 #ifndef HAVE_COGL_GLES /* GLES 1 only has glBlendFunc */
+      if (have_blend_equation_seperate &&
+          material->blend_equation_rgb != material->blend_equation_alpha)
+        GE (glBlendEquationSeparate (material->blend_equation_rgb,
+                                     material->blend_equation_alpha));
+      else
+        GE (glBlendEquation (material->blend_equation_rgb));
+
+      if (blend_factor_uses_constant (material->blend_src_factor_rgb) ||
+          blend_factor_uses_constant (material->blend_src_factor_alpha) ||
+          blend_factor_uses_constant (material->blend_dst_factor_rgb) ||
+          blend_factor_uses_constant (material->blend_dst_factor_alpha))
+        GE (glBlendColor (material->blend_constant[0],
+                          material->blend_constant[1],
+                          material->blend_constant[2],
+                          material->blend_constant[3]));
+
       if (have_blend_func_separate &&
           (material->blend_src_factor_rgb != material->blend_src_factor_alpha ||
            (material->blend_src_factor_rgb !=
             material->blend_src_factor_alpha)))
-        {
-          if (have_blend_equation_seperate &&
-              material->blend_equation_rgb != material->blend_equation_alpha)
-            GE (glBlendEquationSeparate (material->blend_equation_rgb,
-                                         material->blend_equation_alpha));
-          else
-            GE (glBlendEquation (material->blend_equation_rgb));
-
-          GE (glBlendFuncSeparate (material->blend_src_factor_rgb,
-                                   material->blend_dst_factor_rgb,
-                                   material->blend_src_factor_alpha,
-                                   material->blend_dst_factor_alpha));
-          GE (glBlendColor (material->blend_constant[0],
-                            material->blend_constant[1],
-                            material->blend_constant[2],
-                            material->blend_constant[3]));
-        }
+        GE (glBlendFuncSeparate (material->blend_src_factor_rgb,
+                                 material->blend_dst_factor_rgb,
+                                 material->blend_src_factor_alpha,
+                                 material->blend_dst_factor_alpha));
       else
 #endif
-      GE (glBlendFunc (material->blend_src_factor_rgb,
-                       material->blend_dst_factor_rgb));
+        GE (glBlendFunc (material->blend_src_factor_rgb,
+                         material->blend_dst_factor_rgb));
     }
 }
 
