@@ -286,6 +286,9 @@ _cogl_texture_2d_new_from_bitmap (CoglHandle       bmp_handle,
 
   g_return_val_if_fail (bmp_handle != COGL_INVALID_HANDLE, COGL_INVALID_HANDLE);
 
+  internal_format = _cogl_texture_determine_internal_format (bmp->format,
+                                                             internal_format);
+
   if (!_cogl_texture_2d_can_create (bmp->width, bmp->height, internal_format))
     return COGL_INVALID_HANDLE;
 
@@ -347,13 +350,22 @@ _cogl_texture_2d_transform_coords_to_gl (CoglTexture *tex,
      anything */
 }
 
-static gboolean
+static CoglTransformResult
 _cogl_texture_2d_transform_quad_coords_to_gl (CoglTexture *tex,
                                               float *coords)
 {
   /* The texture coordinates map directly so we don't need to do
-     anything */
-  return TRUE;
+     anything other than check for repeats */
+
+  gboolean need_repeat = FALSE;
+  int i;
+
+  for (i = 0; i < 4; i++)
+    if (coords[i] < 0.0f || coords[i] > 1.0f)
+      need_repeat = TRUE;
+
+  return (need_repeat ? COGL_TRANSFORM_HARDWARE_REPEAT
+          : COGL_TRANSFORM_NO_REPEAT);
 }
 
 static gboolean
@@ -484,6 +496,8 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   if (tmp_bmp_owner)
     g_free (tmp_bmp.data);
 
+  tex_2d->mipmaps_dirty = TRUE;
+
   return TRUE;
 }
 
@@ -546,6 +560,9 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
       target_bmp.data = g_malloc (target_bmp.height * target_bmp.rowstride);
     }
 
+  _cogl_texture_driver_prep_gl_for_pixels_download (target_bmp.rowstride,
+                                                    closest_bpp);
+
   GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
   if (!_cogl_texture_driver_gl_get_tex_image (GL_TEXTURE_2D,
                                               closest_gl_format,
